@@ -20,8 +20,8 @@
 #define BZRTP_H
 
 #include <stdint.h>
-#include "bctoolbox/crypto.h"
-#include "bctoolbox/port.h"
+#include "bctoolbox/crypto.h" // for bctbx_rng_context_t
+#include "bctoolbox/port.h" // for bctbx_mutex_t
 
 #ifdef _MSC_VER
 	#ifdef BZRTP_STATIC
@@ -48,9 +48,9 @@
 /**
  * Define different types of crypto functions */
 #define ZRTP_HASH_TYPE			0x01
-#define ZRTP_CIPHERBLOCK_TYPE 		0x02
+#define ZRTP_CIPHERBLOCK_TYPE 	0x02
 #define ZRTP_AUTHTAG_TYPE		0x04
-#define ZRTP_KEYAGREEMENT_TYPE		0x08
+#define ZRTP_KEYAGREEMENT_TYPE	0x08
 #define ZRTP_SAS_TYPE			0x10
 
 /**
@@ -60,8 +60,9 @@
 
 #define	ZRTP_HASH_S256			0x11
 #define	ZRTP_HASH_S384			0x12
-#define	ZRTP_HASH_N256			0x13
-#define	ZRTP_HASH_N384			0x14
+#define	ZRTP_HASH_S512			0x13
+#define	ZRTP_HASH_N256			0x14
+#define	ZRTP_HASH_N384			0x15
 
 #define ZRTP_CIPHER_AES1		0x21
 #define ZRTP_CIPHER_AES2		0x22
@@ -81,17 +82,31 @@
  */
 #define ZRTP_KEYAGREEMENT_DH2k	0x41
 #define ZRTP_KEYAGREEMENT_X255	0x42
-#define ZRTP_KEYAGREEMENT_EC25	0x43
-#define ZRTP_KEYAGREEMENT_X448	0x44
-#define ZRTP_KEYAGREEMENT_DH3k	0x45
-#define ZRTP_KEYAGREEMENT_EC38	0x46
-#define ZRTP_KEYAGREEMENT_EC52	0x47
+#define ZRTP_KEYAGREEMENT_K255	0x43
+#define ZRTP_KEYAGREEMENT_EC25	0x44
+#define ZRTP_KEYAGREEMENT_X448	0x45
+#define ZRTP_KEYAGREEMENT_K448	0x46
+#define ZRTP_KEYAGREEMENT_DH3k	0x47
+#define ZRTP_KEYAGREEMENT_EC38	0x48
+#define ZRTP_KEYAGREEMENT_EC52	0x49
+#define ZRTP_KEYAGREEMENT_KYB1	0x4a
+#define ZRTP_KEYAGREEMENT_KYB2	0x4b
+#define ZRTP_KEYAGREEMENT_KYB3	0x4c
+#define ZRTP_KEYAGREEMENT_HQC1	0x4d
+#define ZRTP_KEYAGREEMENT_HQC2	0x4e
+#define ZRTP_KEYAGREEMENT_HQC3	0x4f
+#define ZRTP_KEYAGREEMENT_K255_KYB512	0x51
+#define ZRTP_KEYAGREEMENT_K255_HQC128	0x52
+#define ZRTP_KEYAGREEMENT_K448_KYB1024	0x53
+#define ZRTP_KEYAGREEMENT_K448_HQC256	0x54
+#define ZRTP_KEYAGREEMENT_K255_KYB512_HQC128	0x55
+#define ZRTP_KEYAGREEMENT_K448_KYB1024_HQC256	0x56
 
-#define ZRTP_KEYAGREEMENT_Prsh	0x4e
-#define ZRTP_KEYAGREEMENT_Mult	0x4f
+#define ZRTP_KEYAGREEMENT_Prsh	0x9e
+#define ZRTP_KEYAGREEMENT_Mult	0x9f
 
-#define ZRTP_SAS_B32			0x51
-#define ZRTP_SAS_B256			0x52
+#define ZRTP_SAS_B32			0xa1
+#define ZRTP_SAS_B256			0xa2
 
 
 /**
@@ -122,23 +137,33 @@ typedef struct bzrtpSrtpSecrets_struct  {
 	uint8_t sasAlgo; /**< The SAS rendering algo selected during ZRTP negotiation */
 	uint8_t cacheMismatch; /**< Flag set to 1 in case of ZRTP cache mismatch, may occurs only on first channel(the one computing SAS) */
 	uint8_t auxSecretMismatch; /**< Flag set to BZRTP_AUXSECRET_MATCH, BZRTP_AUXSECRET_MISMATCH or BZRTP_AUXSECRET_UNSET, may occurs only on first channel(the one computing SAS), in case of mismatch it may be ignored and we can still validate the SAS */
+	uint8_t peerAcceptGoClear; /**< Flag set to 1 in case of peer accept receiving GoClear */
 } bzrtpSrtpSecrets_t;
 
+/* define flag IDs */
+#define BZRTP_IS_INITIALISED            0x00
+#define BZRTP_IS_SECURE                 0x01
+#define BZRTP_PEER_SUPPORT_MULTICHANNEL 0x02
+#define BZRTP_SELF_ACCEPT_GOCLEAR       0x03
+#define BZRTP_PEER_ACCEPT_GOCLEAR       0x04
+
 /* define auxSecretMismatch flag codes */
 #define BZRTP_AUXSECRET_MATCH		0x00
 #define BZRTP_AUXSECRET_MISMATCH	0x01
 #define BZRTP_AUXSECRET_UNSET		0x02
 
 /* define message levels */
-#define BZRTP_MESSAGE_ERROR	0x00
+#define BZRTP_MESSAGE_ERROR     0x00
 #define BZRTP_MESSAGE_WARNING	0x01
-#define BZRTP_MESSAGE_LOG	0x02
-#define BZRTP_MESSAGE_DEBUG	0x03
+#define BZRTP_MESSAGE_LOG       0x02
+#define BZRTP_MESSAGE_DEBUG     0x03
 
 /* define message codes */
 #define BZRTP_MESSAGE_CACHEMISMATCH 		0x01
 #define BZRTP_MESSAGE_PEERVERSIONOBSOLETE	0x02
-#define BZRTP_MESSAGE_PEERNOTBZRTP		0x03
+#define BZRTP_MESSAGE_PEERNOTBZRTP          0x03
+#define BZRTP_MESSAGE_PEERREQUESTGOCLEAR    0x04
+#define BZRTP_MESSAGE_PEERACKGOCLEAR		0x05
 
 /**
  * Function pointer used by bzrtp to free memory allocated by callbacks.
@@ -176,21 +201,31 @@ typedef struct bzrtpCallbacks_struct {
 #define BZRTP_ERROR_OUTPUTBUFFER_LENGTH				0x0040
 #define BZRTP_ERROR_HELLOHASH_MISMATCH				0x0080
 #define BZRTP_ERROR_CHANNELALREADYSTARTED			0x0100
-#define BZRTP_ERROR_CACHEDISABLED				0x0200
+#define BZRTP_ERROR_CACHEDISABLED					0x0200
 #define BZRTP_ERROR_CACHEMIGRATIONFAILED			0x0400
 #define BZRTP_ERROR_CACHE_PEERNOTFOUND				0x0800
+#define BZRTP_ERROR_INVALIDCLEARMAC                 0x1000
+#define BZRTP_ERROR_PEERDOESNTACCEPTGOCLEAR         0x2000
+#define BZRTP_ERROR_GOCLEARDISABLED					0x4000
+#define BZRTP_ERROR_INVALIDARGUMENT					0x8000
 
 /* channel status definition */
 #define BZRTP_CHANNEL_NOTFOUND						0x1000
 #define BZRTP_CHANNEL_INITIALISED					0x1001
 #define BZRTP_CHANNEL_ONGOING						0x1002
 #define BZRTP_CHANNEL_SECURE						0x1004
-#define BZRTP_CHANNEL_ERROR						0x1008
+#define BZRTP_CHANNEL_CLEAR                         0x1010
+#define BZRTP_CHANNEL_ERROR                         0x1008
 
 /* role mapping */
 #define BZRTP_ROLE_INITIATOR	0
 #define	BZRTP_ROLE_RESPONDER	1
 
+/* channel receiving GoClear message */
+#define BZRTP_RECEPTION_UNKNOWN 0
+#define BZRTP_RECEPTION_YES     1
+#define BZRTP_RECEPTION_NO      2
+
 /* cache related value */
 #define BZRTP_CACHE_SETUP		0x2000
 #define BZRTP_CACHE_UPDATE		0x2001
@@ -237,8 +272,6 @@ BZRTP_EXPORT bzrtpContext_t *bzrtp_createBzrtpContext(void);
  */
 BZRTP_EXPORT int bzrtp_initBzrtpContext(bzrtpContext_t *context, uint32_t selfSSRC);
 
-BZRTP_EXPORT void bzrtp_resetBzrtpContext(bzrtpContext_t *context);
-
 /**
  * Free memory of context structure to a channel, if all channels are freed, free the global zrtp context
  * @param[in]	context		Context hosting the channel to be destroyed.(note: the context zrtp context itself is destroyed with the last channel)
@@ -364,7 +397,7 @@ BZRTP_EXPORT void bzrtp_resetSASVerified(bzrtpContext_t *zrtpContext);
  * @param[in,out]	zrtpContext				The ZRTP context we're dealing with
  * @param[in]		selfSSRC				The SSRC identifying the channel to reset
  *
- * return 0 on success, error code otherwise
+ * @return 0 on success, error code otherwise
  */
 BZRTP_EXPORT int bzrtp_resetRetransmissionTimer(bzrtpContext_t *zrtpContext, uint32_t selfSSRC);
 
@@ -380,14 +413,25 @@ BZRTP_EXPORT int bzrtp_resetRetransmissionTimer(bzrtpContext_t *zrtpContext, uin
 BZRTP_EXPORT uint8_t bzrtp_getSupportedCryptoTypes(bzrtpContext_t *zrtpContext, uint8_t algoType, uint8_t supportedTypes[7]);
 
 /**
- * @brief set the supported crypto types
+ * @brief set the supported crypto types. This function must be called before the context is initialised, just after creation.
  *
  * @param[in,out]	zrtpContext				The ZRTP context we're dealing with
  * @param[in]		algoType				mapped to defines, must be in [ZRTP_HASH_TYPE, ZRTP_CIPHERBLOCK_TYPE, ZRTP_AUTHTAG_TYPE, ZRTP_KEYAGREEMENT_TYPE or ZRTP_SAS_TYPE]
  * @param[in]		supportedTypes			mapped to uint8_t value of the 4 char strings giving the supported types as string according to rfc section 5.1.2 to 5.1.6
  * @param[in]		supportedTypesCount		number of supported crypto types
+ *
+ * @return 0 on success, error code otherwise
  */
-BZRTP_EXPORT void bzrtp_setSupportedCryptoTypes(bzrtpContext_t *zrtpContext, uint8_t algoType, uint8_t supportedTypes[7], uint8_t supportedTypesCount);
+BZRTP_EXPORT int bzrtp_setSupportedCryptoTypes(bzrtpContext_t *zrtpContext, uint8_t algoType, uint8_t supportedTypes[7], uint8_t supportedTypesCount);
+
+/**
+ * @brief Set the selfAcceptGoClear flag
+ *
+ * @param[in,out]   zrtpContext The ZRTP context we're dealing with
+ * @param[in]       flagId      mapped to defines, must be BZRTP_SELF_ACCEPT_GOCLEAR
+ * @param[in]       value       Flag value
+ */
+BZRTP_EXPORT int bzrtp_setFlags(bzrtpContext_t *zrtpContext, uint8_t flagId, uint8_t value);
 
 /**
  * @brief Set the peer hello hash given by signaling to a ZRTP channel
@@ -642,6 +686,98 @@ BZRTP_EXPORT int bzrtp_exportKey(bzrtpContext_t *zrtpContext, char *label, size_
  */
 BZRTP_EXPORT int bzrtp_cache_getPeerStatus_lock(void *dbPointer, const char *peerURI, bctbx_mutex_t *zidCacheMutex);
 
+/**
+ * @brief	Retrieve the name of the algo in string
+ *
+ * @param[in]	algo	Id of the algo
+ *
+ * @return	The of the algo in string
+ */
+BZRTP_EXPORT const char *bzrtp_algoToString(uint8_t algo);
+
+/**
+ * @brief set the maximum size of a ZRTP packet generated locally
+ * MTU must be at least 600 bytes to avoid useless fragmentation of small packets
+ *
+ * @param[in]		zrtpContext		The ZRTP context we're dealing with
+ * @param[in]		mtu 			The size in bytes of the maximum allowed for a ZRTP packet. If this parameter is less than 600, the actual MTU is set to 600
+ *
+ * @return 0 on succes, error code otherwise
+ */
+BZRTP_EXPORT int bzrtp_set_MTU(bzrtpContext_t *zrtpContext, size_t mtu);
+
+/**
+ * @brief get the maximum size of a ZRTP packet generated locally
+ *
+ * @param[in]		zrtpContext		The ZRTP context we're dealing with
+ *
+ * @return the maximum size in bytes of a ZRTP packet generated locally
+ */
+BZRTP_EXPORT size_t bzrtp_get_MTU(bzrtpContext_t *zrtpContext);
+
+
+/**
+ * @brief Retrieve the list of available key agreements algorithms
+ *
+ * @param[in/out] algos		an array containing the list of available algorithms mapped on uint8
+ * 				as defined in this header(ZRTP_KEYAGREEMENT_<ID>). Caller is responsible for the array allocation
+ *
+ * @return The number of available key agreement algorithms
+ */
+BZRTP_EXPORT uint8_t bzrtp_available_key_agreement(uint8_t availableTypes[256]);
+
+
+/**
+ * @brief check is Post Quantum algorithms are available
+ * @return TRUE when PQ key exchange algorithms are available, FALSE otherwise
+ */
+BZRTP_EXPORT bool_t bzrtp_is_PQ_available(void);
+
+/**
+ * @brief Create a GoClear event and send it to the state machine
+ * The user is in secure state.
+ * He decided to change his encryption mode by clicking on a button for example.
+ * The end point continues to send SRTP packets.
+ * On ClearACK reception the end point deletes all key materials.
+ *
+ * @param[in] context	The ZRTP context we're dealing with
+ * @param[in] selfSSRC	The SSRC identifying the channel
+ *
+ * @return one of :
+ *  - BZRTP_ERROR_INVALIDCONTEXT : The context is invalid or the channel is not in secure state
+ *  - Return value of the state machine
+ */
+BZRTP_EXPORT int bzrtp_sendGoClear(bzrtpContext_t *context, uint32_t selfSSRC);
+
+/**
+ * @brief Create a acceptGoClear event and send it to the state machine
+ * The user received a valid GoClear packet and sent a ClearACK message (so the end point stops to send SRTP packets).
+ * He agrees to change the encryption mode by clicking on a button for example.
+ * The the sending of RTP packets may begin.
+ *
+ * @param[in] context	The ZRTP context we're dealing with
+ * @param[in] selfSSRC	The SSRC identifying the channel
+ *
+ * @return one of :
+ *  - BZRTP_ERROR_INVALIDCONTEXT : The context is invalid
+ *  - Return value of the state machine
+ */
+BZRTP_EXPORT int bzrtp_confirmGoClear(bzrtpContext_t *zrtpContext, uint32_t selfSSRC);
+
+/**
+ * @brief Create a BackToSecure event and send it to the state machine
+ * The user has a clear channel.
+ * He decided to resume the secure mode by clicking on a button for example.
+ *
+ * @param zrtpContext	The ZRTP context we're dealing with
+ * @param selfSSRC		The SSRC identifying the channel
+ *
+ * @return
+ *	- BZRTP_ERROR_INVALIDCONTEXT : The context is invalid
+ *	- Return value of the state machine
+ */
+BZRTP_EXPORT int bzrtp_backToSecureMode(bzrtpContext_t *zrtpContext, uint32_t selfSSRC);
+
 #ifdef __cplusplus
 }
 #endif
