@@ -45,7 +45,7 @@ static uint8_t copyCryptoTypes(uint8_t destination[7], uint8_t source[7], uint8_
 bzrtpContext_t *bzrtp_createBzrtpContext(void) {
 	int i;
 	/*** create and intialise the context structure ***/
-	bzrtpContext_t *context = malloc(sizeof(bzrtpContext_t));
+	bzrtpContext_t *context = (bzrtpContext_t *)malloc(sizeof(bzrtpContext_t));
 	memset(context, 0, sizeof(bzrtpContext_t));
 
 	/* start the random number generator */
@@ -58,6 +58,10 @@ bzrtpContext_t *bzrtp_createBzrtpContext(void) {
 	context->isSecure = 0; /* start unsecure */
 	context->peerSupportMultiChannel = 0; /* peer does not support Multichannel by default */
 	context->isInitialised = 0; /* will be set by bzrtp_initBzrtpContext */
+#ifdef GOCLEAR_ENABLED
+	context->selfAcceptGoClear = 0;
+	context->peerAcceptGoClear = 0;
+#endif /* GOCLEAR_ENABLED */
 
 	/* set to NULL all callbacks pointer */
 	context->zrtpCallbacks.bzrtp_statusMessage = NULL;
@@ -107,6 +111,9 @@ bzrtpContext_t *bzrtp_createBzrtpContext(void) {
 	context->exportedKey = NULL;
 	context->exportedKeyLength = 0;
 
+	/* initialise MTU to default value */
+	context->mtu = BZRTP_DEFAULT_MTU;
+
 	return context;
 }
 
@@ -174,17 +181,6 @@ int bzrtp_setZIDCache_lock(bzrtpContext_t *context, void *zidCache, const char *
 #endif /* ZIDCACHE_ENABLED */
 }
 
-void bzrtp_resetBzrtpContext(bzrtpContext_t *context) {
-	int i;
-	for (i=0; i<ZRTP_MAX_CHANNEL_NUMBER; i++) {
-		if (context->channelContext[i]) {
-			void * clientData = context->channelContext[i]->clientData;
-			bzrtp_initChannelContext(context, context->channelContext[i], context->channelContext[i]->selfSSRC, context->channelContext[i]->isMainChannel);
-			context->channelContext[i]->clientData = clientData;
-		}
-	}
-}
-
 /**
  * @brief Perform some initialisation which can't be done without some callback functions:
  *  This function is called once per session when the first channel is created.
@@ -207,6 +203,35 @@ int bzrtp_initBzrtpContext(bzrtpContext_t *context, uint32_t selfSSRC) {
 	bzrtp_getSelfZID_lock(context->zidCache, context->selfURI, context->selfZID, context->RNGContext, context->zidCacheMutex);
 	context->isInitialised = 1;
 
+	/* In the case where we supporte a post quantum algorithm,
+	 * we need to supporte AES3 and SHA512 */
+	int i = 0;
+	while (i < context->kc && !bzrtp_isPostQuantum(context->supportedKeyAgreement[i])) {
+		i++;
+	}
+	if (i < context->kc) {
+		int j = 0;
+		if (context->cc < 7) {
+			while (context->supportedCipher[j] != ZRTP_CIPHER_AES3 && j < context->cc) {
+				j++;
+			}
+			if (j == context->cc) {
+				context->supportedCipher[context->cc] = ZRTP_CIPHER_AES3;
+				context->cc++;
+			}
+		}
+		j = 0;
+		if (context->hc < 7) {
+			while (context->supportedHash[j] != ZRTP_HASH_S512 && j < context->hc) {
+				j++;
+			}
+			if (j == context->hc) {
+				context->supportedHash[context->hc] = ZRTP_HASH_S512;
+				context->hc++;
+			}
+		}
+	}
+
 	/* allocate 1 channel context, set all the others pointers to NULL */
 	context->channelContext[0] = (bzrtpChannelContext_t *)malloc(sizeof(bzrtpChannelContext_t));
 	memset(context->channelContext[0], 0, sizeof(bzrtpChannelContext_t));
@@ -249,10 +274,13 @@ int bzrtp_destroyBzrtpContext(bzrtpContext_t *context, uint32_t selfSSRC) {
 	/* We have no more channel, destroy the zrtp context */
 	/* key agreement context shall already been destroyed after s0 computation, but just in case */
 	if (context->keyAgreementContext != NULL) {
-		if (context->keyAgreementAlgo == ZRTP_KEYAGREEMENT_X255 || context->keyAgreementAlgo == ZRTP_KEYAGREEMENT_X448) {
+		if (bzrtp_isKem(context->keyAgreementAlgo)) {
+			bzrtp_destroyKEMContext((bzrtp_KEMContext_t *)context->keyAgreementContext);
+		}
+		else if (context->keyAgreementAlgo == ZRTP_KEYAGREEMENT_X255 || context->keyAgreementAlgo == ZRTP_KEYAGREEMENT_X448) {
 			bctbx_DestroyECDHContext((bctbx_ECDHContext_t *)context->keyAgreementContext);
 		}
-		if (context->keyAgreementAlgo == ZRTP_KEYAGREEMENT_DH2k || context->keyAgreementAlgo == ZRTP_KEYAGREEMENT_DH3k) {
+		else if (context->keyAgreementAlgo == ZRTP_KEYAGREEMENT_DH2k || context->keyAgreementAlgo == ZRTP_KEYAGREEMENT_DH3k) {
 			bctbx_DestroyDHMContext((bctbx_DHMContext_t *)context->keyAgreementContext);
 		}
 		context->keyAgreementContext = NULL;
@@ -407,11 +435,6 @@ int bzrtp_startChannelEngine(bzrtpContext_t *zrtpContext, uint32_t selfSSRC) {
 		if (zrtpContext->isSecure == 0) {
 			return BZRTP_ERROR_CONTEXTNOTREADY;
 		}
-
-		/* check the peer support Multichannel(shall be set in the first Hello message received) */
-		if (zrtpContext->peerSupportMultiChannel == 0) {
-			return BZRTP_ERROR_MULTICHANNELNOTSUPPORTEDBYPEER;
-		}
 	}
 
 	/* set the timer reference to 0 to force a message to be sent at first timer tick */
@@ -525,25 +548,32 @@ int bzrtp_processMessage(bzrtpContext_t *zrtpContext, uint32_t selfSSRC, uint8_t
 	}
 
 	/* first check the packet */
-	zrtpPacket = bzrtp_packetCheck(zrtpPacketString, zrtpPacketStringLength, zrtpChannelContext->peerSequenceNumber, &retval);
+	uint8_t *incomingPacket = zrtpPacketString;
+	uint16_t incomingPacketLength = zrtpPacketStringLength;
+	zrtpPacket = bzrtp_packetCheck(&incomingPacket, &incomingPacketLength, zrtpChannelContext, &retval);
 	if (retval != 0) {
+		if (retval == BZRTP_PARSER_INFO_PACKETFRAGMENT) {
+			/* fragment of incomplete packet incoming, just wait for the rest of it to arrive*/
+			return 0;
+		}
 		/*TODO: check the returned error code and do something or silent drop? */
 		return retval;
 	}
 
-	/* TODO: Intercept error and ping zrtp packets */
+	/* Intercept error(Not managed yet) and ping zrtp packets */
 	/* if we have a ping packet, just answer with a ping ACK and do not forward to the state machine */
 	if (zrtpPacket->messageType == MSGTYPE_PING) {
 		bzrtpPacket_t *pingAckPacket = NULL;
 
-		bzrtp_packetParser(zrtpContext, zrtpChannelContext, zrtpPacketString, zrtpPacketStringLength, zrtpPacket);
+		bzrtp_packetParser(zrtpContext, zrtpChannelContext, incomingPacket, incomingPacketLength, zrtpPacket);
 		/* store ping packet in the channel context as packet creator will need it to create the pingACK */
 		zrtpChannelContext->pingPacket = zrtpPacket;
 		/* create the pingAck packet */
 		pingAckPacket = bzrtp_createZrtpPacket(zrtpContext, zrtpChannelContext, MSGTYPE_PINGACK, &retval);
 		if (retval == 0) {
-			retval = bzrtp_packetBuild(zrtpContext, zrtpChannelContext, pingAckPacket, zrtpChannelContext->selfSequenceNumber);
+			retval = bzrtp_packetBuild(zrtpContext, zrtpChannelContext, pingAckPacket);
 			if (retval==0 && zrtpContext->zrtpCallbacks.bzrtp_sendData!=NULL) { /* send the packet */
+				bzrtp_packetSetSequenceNumber(pingAckPacket, zrtpChannelContext->selfSequenceNumber);
 				zrtpContext->zrtpCallbacks.bzrtp_sendData(zrtpChannelContext->clientData, pingAckPacket->packetString, pingAckPacket->messageLength+ZRTP_PACKET_OVERHEAD);
 				zrtpChannelContext->selfSequenceNumber++;
 			}
@@ -559,8 +589,8 @@ int bzrtp_processMessage(bzrtpContext_t *zrtpContext, uint32_t selfSSRC, uint8_t
 
 	/* build a packet event of it and send it to the state machine */
 	event.eventType = BZRTP_EVENT_MESSAGE;
-	event.bzrtpPacketString = zrtpPacketString;
-	event.bzrtpPacketStringLength = zrtpPacketStringLength;
+	event.bzrtpPacketString = incomingPacket;
+	event.bzrtpPacketStringLength = incomingPacketLength;
 	event.bzrtpPacket = zrtpPacket;
 	event.zrtpContext = zrtpContext;
 	event.zrtpChannelContext = zrtpChannelContext;
@@ -636,11 +666,11 @@ int bzrtp_exportKey(bzrtpContext_t *zrtpContext, char *label, size_t labelLength
 			*derivedKeyLength = zrtpChannelContext->hashLength;
 		}
 
-		bzrtp_keyDerivationFunction(zrtpChannelContext->s0, zrtpChannelContext->hashLength, (uint8_t *)label, labelLength, zrtpChannelContext->KDFContext, zrtpChannelContext->KDFContextLength, *derivedKeyLength, zrtpChannelContext->hmacFunction, derivedKey);
+		bzrtp_keyDerivationFunction(zrtpChannelContext->s0, zrtpChannelContext->hashLength, (uint8_t *)label, labelLength, zrtpChannelContext->KDFContext, zrtpChannelContext->KDFContextLength, (uint_t)(*derivedKeyLength), zrtpChannelContext->hmacFunction, derivedKey);
 #else /* SUPPORT_EXPORTEDKEY_V010000 */
 		/* We do not support anymore backward compatibility, just do nothing but send an error message*/
 		if (zrtpContext->zrtpCallbacks.bzrtp_statusMessage!=NULL && zrtpContext->zrtpCallbacks.bzrtp_messageLevel>=BZRTP_MESSAGE_ERROR) { /* use error level as we explicitely compile with no support for older version */
-				zrtpContext->zrtpCallbacks.bzrtp_statusMessage(zrtpChannelContext->clientData, BZRTP_MESSAGE_ERROR, BZRTP_MESSAGE_PEERVERSIONOBSOLETE, "obsolete bzrtp version are not supported anymore");
+			zrtpContext->zrtpCallbacks.bzrtp_statusMessage(zrtpChannelContext->clientData, BZRTP_MESSAGE_ERROR, BZRTP_MESSAGE_PEERVERSIONOBSOLETE, "obsolete bzrtp version are not supported anymore");
 		}
 #endif /* SUPPORT_EXPORTEDKEY_V010000 */
 	} else { /* peer either use version 1.1 of BZRTP or another library, just stick to the RFC to create the export key */
@@ -660,7 +690,7 @@ int bzrtp_exportKey(bzrtpContext_t *zrtpContext, char *label, size_t labelLength
 			*derivedKeyLength = zrtpChannelContext->hashLength;
 		}
 
-		bzrtp_keyDerivationFunction(zrtpContext->exportedKey, zrtpChannelContext->hashLength, (uint8_t *)label, labelLength, zrtpChannelContext->KDFContext, zrtpChannelContext->KDFContextLength, *derivedKeyLength, zrtpChannelContext->hmacFunction, derivedKey);
+		bzrtp_keyDerivationFunction(zrtpContext->exportedKey, zrtpChannelContext->hashLength, (uint8_t *)label, labelLength, zrtpChannelContext->KDFContext, zrtpChannelContext->KDFContextLength, (uint8_t)(*derivedKeyLength), zrtpChannelContext->hmacFunction, derivedKey);
 
 	}
 	return 0;
@@ -717,18 +747,18 @@ uint8_t bzrtp_getSupportedCryptoTypes(bzrtpContext_t *zrtpContext, uint8_t algoT
 	}
 
 	switch(algoType) {
-		case ZRTP_HASH_TYPE:
-			return copyCryptoTypes(supportedTypes, zrtpContext->supportedHash, zrtpContext->hc);
-		case ZRTP_CIPHERBLOCK_TYPE:
-			return copyCryptoTypes(supportedTypes, zrtpContext->supportedCipher, zrtpContext->cc);
-		case ZRTP_AUTHTAG_TYPE:
-			return copyCryptoTypes(supportedTypes, zrtpContext->supportedAuthTag, zrtpContext->ac);
-		case ZRTP_KEYAGREEMENT_TYPE:
-			return copyCryptoTypes(supportedTypes, zrtpContext->supportedKeyAgreement, zrtpContext->kc);
-		case ZRTP_SAS_TYPE:
-			return copyCryptoTypes(supportedTypes, zrtpContext->supportedSas, zrtpContext->sc);
-		default:
-			return 0;
+	case ZRTP_HASH_TYPE:
+		return copyCryptoTypes(supportedTypes, zrtpContext->supportedHash, zrtpContext->hc);
+	case ZRTP_CIPHERBLOCK_TYPE:
+		return copyCryptoTypes(supportedTypes, zrtpContext->supportedCipher, zrtpContext->cc);
+	case ZRTP_AUTHTAG_TYPE:
+		return copyCryptoTypes(supportedTypes, zrtpContext->supportedAuthTag, zrtpContext->ac);
+	case ZRTP_KEYAGREEMENT_TYPE:
+		return copyCryptoTypes(supportedTypes, zrtpContext->supportedKeyAgreement, zrtpContext->kc);
+	case ZRTP_SAS_TYPE:
+		return copyCryptoTypes(supportedTypes, zrtpContext->supportedSas, zrtpContext->sc);
+	default:
+		return 0;
 	}
 }
 
@@ -740,39 +770,68 @@ uint8_t bzrtp_getSupportedCryptoTypes(bzrtpContext_t *zrtpContext, uint8_t algoT
  * @param[in]		supportedTypes			mapped to uint8_t value of the 4 char strings giving the supported types as string according to rfc section 5.1.2 to 5.1.6
  * @param[in]		supportedTypesCount		number of supported crypto types
  */
-void bzrtp_setSupportedCryptoTypes(bzrtpContext_t *zrtpContext, uint8_t algoType, uint8_t supportedTypes[7], uint8_t supportedTypesCount)
-{
-	uint8_t implementedTypes[7];
+int bzrtp_setSupportedCryptoTypes(bzrtpContext_t *zrtpContext, uint8_t algoType, uint8_t supportedTypes[7], uint8_t supportedTypesCount) {
+	if (zrtpContext==NULL) {
+		return BZRTP_ERROR_INVALIDCONTEXT;
+	}
+
+	/* if the context is already initialised, it's too late to set the supported crypto algos */
+	if (zrtpContext->isInitialised) {
+		return BZRTP_ERROR_CONTEXTNOTREADY;
+	}
+
+	uint8_t implementedTypes[256];
 	uint8_t implementedTypesCount;
 
-	if (zrtpContext==NULL) {
-		return;
-	}
-
-	implementedTypesCount = bzrtpUtils_getAvailableCryptoTypes(algoType, implementedTypes);
+	implementedTypesCount = bzrtpUtils_getAllAvailableCryptoTypes(algoType, implementedTypes);
 
 	switch(algoType) {
-		case ZRTP_HASH_TYPE:
-			zrtpContext->hc = selectCommonAlgo(supportedTypes, supportedTypesCount, implementedTypes, implementedTypesCount, zrtpContext->supportedHash);
-			bzrtp_addMandatoryCryptoTypesIfNeeded(algoType, zrtpContext->supportedHash, &zrtpContext->hc);
-			break;
-		case ZRTP_CIPHERBLOCK_TYPE:
-			zrtpContext->cc = selectCommonAlgo(supportedTypes, supportedTypesCount, implementedTypes, implementedTypesCount, zrtpContext->supportedCipher);
-			bzrtp_addMandatoryCryptoTypesIfNeeded(algoType, zrtpContext->supportedCipher, &zrtpContext->cc);
-			break;
-		case ZRTP_AUTHTAG_TYPE:
-			zrtpContext->ac = selectCommonAlgo(supportedTypes, supportedTypesCount, implementedTypes, implementedTypesCount, zrtpContext->supportedAuthTag);
-			bzrtp_addMandatoryCryptoTypesIfNeeded(algoType, zrtpContext->supportedAuthTag, &zrtpContext->ac);
-			break;
-		case ZRTP_KEYAGREEMENT_TYPE:
-			zrtpContext->kc = selectCommonAlgo(supportedTypes, supportedTypesCount, implementedTypes, implementedTypesCount, zrtpContext->supportedKeyAgreement);
-			bzrtp_addMandatoryCryptoTypesIfNeeded(algoType, zrtpContext->supportedKeyAgreement, &zrtpContext->kc);
-			break;
-		case ZRTP_SAS_TYPE:
-			zrtpContext->sc = selectCommonAlgo(supportedTypes, supportedTypesCount, implementedTypes, implementedTypesCount, zrtpContext->supportedSas);
-			bzrtp_addMandatoryCryptoTypesIfNeeded(algoType, zrtpContext->supportedSas, &zrtpContext->sc);
-			break;
+	case ZRTP_HASH_TYPE:
+		zrtpContext->hc = bzrtp_selectCommonAlgo(supportedTypes, supportedTypesCount, implementedTypes, implementedTypesCount, zrtpContext->supportedHash);
+		bzrtp_addMandatoryCryptoTypesIfNeeded(algoType, zrtpContext->supportedHash, &zrtpContext->hc);
+		break;
+	case ZRTP_CIPHERBLOCK_TYPE:
+		zrtpContext->cc = bzrtp_selectCommonAlgo(supportedTypes, supportedTypesCount, implementedTypes, implementedTypesCount, zrtpContext->supportedCipher);
+		bzrtp_addMandatoryCryptoTypesIfNeeded(algoType, zrtpContext->supportedCipher, &zrtpContext->cc);
+		break;
+	case ZRTP_AUTHTAG_TYPE:
+		zrtpContext->ac = bzrtp_selectCommonAlgo(supportedTypes, supportedTypesCount, implementedTypes, implementedTypesCount, zrtpContext->supportedAuthTag);
+		bzrtp_addMandatoryCryptoTypesIfNeeded(algoType, zrtpContext->supportedAuthTag, &zrtpContext->ac);
+		break;
+	case ZRTP_KEYAGREEMENT_TYPE:
+		zrtpContext->kc = bzrtp_selectCommonAlgo(supportedTypes, supportedTypesCount, implementedTypes, implementedTypesCount, zrtpContext->supportedKeyAgreement);
+		bzrtp_addMandatoryCryptoTypesIfNeeded(algoType, zrtpContext->supportedKeyAgreement, &zrtpContext->kc);
+		break;
+	case ZRTP_SAS_TYPE:
+		zrtpContext->sc = bzrtp_selectCommonAlgo(supportedTypes, supportedTypesCount, implementedTypes, implementedTypesCount, zrtpContext->supportedSas);
+		bzrtp_addMandatoryCryptoTypesIfNeeded(algoType, zrtpContext->supportedSas, &zrtpContext->sc);
+		break;
 	}
+
+	return 0;
+}
+
+
+int bzrtp_setFlags(bzrtpContext_t *zrtpContext, uint8_t flagId, uint8_t value) {
+#ifdef GOCLEAR_ENABLED
+	if (zrtpContext == NULL) {
+		return BZRTP_ERROR_INVALIDCONTEXT;
+	}
+
+	switch(flagId){
+		case BZRTP_SELF_ACCEPT_GOCLEAR :
+			if (value == 0) {
+				zrtpContext->selfAcceptGoClear = 0;
+			} else {
+				zrtpContext->selfAcceptGoClear = 1;
+			}
+			return 0;
+		default:
+			return BZRTP_ERROR_INVALIDARGUMENT;
+	}
+#else
+	return BZRTP_ERROR_GOCLEARDISABLED;
+#endif /* GOCLEAR_ENABLED */
 }
 
 /**
@@ -818,16 +877,16 @@ int bzrtp_setPeerHelloHash(bzrtpContext_t *zrtpContext, uint32_t selfSSRC, uint8
 	zrtpChannelContext->peerHelloHash = (uint8_t *)malloc(hexHashStringLength/2*sizeof(uint8_t));
 
 	/* convert to uint8 the hex string */
-	bzrtp_strToUint8(zrtpChannelContext->peerHelloHash, hexHashString, hexHashStringLength);
+	bctbx_str_to_uint8(zrtpChannelContext->peerHelloHash, hexHashString, hexHashStringLength);
 
 	/* Do we already have the peer Hello packet, if yes, check it match the hash */
 	if (zrtpChannelContext->peerPackets[HELLO_MESSAGE_STORE_ID] != NULL) {
 		uint8_t computedPeerHelloHash[32];
 		/* compute hash using implicit hash function: SHA256, skip packet header in the packetString buffer as the hash must be computed on message only */
 		bctbx_sha256(zrtpChannelContext->peerPackets[HELLO_MESSAGE_STORE_ID]->packetString+ZRTP_PACKET_HEADER_LENGTH,
-			zrtpChannelContext->peerPackets[HELLO_MESSAGE_STORE_ID]->messageLength,
-			32,
-			computedPeerHelloHash);
+					 zrtpChannelContext->peerPackets[HELLO_MESSAGE_STORE_ID]->messageLength,
+					 32,
+					 computedPeerHelloHash);
 
 		/* check they are the same */
 		if (memcmp(computedPeerHelloHash, zrtpChannelContext->peerHelloHash, 32)!=0) {
@@ -953,16 +1012,16 @@ int bzrtp_getSelfHelloHash(bzrtpContext_t *zrtpContext, uint32_t selfSSRC, uint8
 
 	/* compute hash using implicit hash function: SHA256, skip packet header in the packetString buffer as the hash must be computed on message only */
 	bctbx_sha256(zrtpChannelContext->selfPackets[HELLO_MESSAGE_STORE_ID]->packetString+ZRTP_PACKET_HEADER_LENGTH,
-			zrtpChannelContext->selfPackets[HELLO_MESSAGE_STORE_ID]->messageLength,
-			32,
-			helloHash);
+				 zrtpChannelContext->selfPackets[HELLO_MESSAGE_STORE_ID]->messageLength,
+				 32,
+				 helloHash);
 
 	/* add version header */
 	strcpy((char *)output, ZRTP_VERSION);
 	output[strlen(ZRTP_VERSION)]=' ';
 
 	/* convert hash to hex string and set it in the output buffer */
-	bzrtp_int8ToStr(output+strlen(ZRTP_VERSION)+1, helloHash, 32);
+	bctbx_int8_to_str(output+strlen(ZRTP_VERSION)+1, helloHash, 32);
 
 	/* add NULL termination */
 	output[strlen(ZRTP_VERSION)+1+64]='\0';
@@ -1044,7 +1103,11 @@ int bzrtp_getChannelStatus(bzrtpContext_t *zrtpContext, uint32_t selfSSRC) {
 	if (zrtpChannelContext->isSecure == 1) {
 		return BZRTP_CHANNEL_SECURE;
 	}
-
+#ifdef GOCLEAR_ENABLED
+	if (zrtpChannelContext->isClear == 1) {
+		return BZRTP_CHANNEL_CLEAR;
+	}
+#endif /* GOCLEAR_ENABLED */
 	return BZRTP_CHANNEL_ONGOING;
 }
 
@@ -1109,6 +1172,10 @@ static int bzrtp_initChannelContext(bzrtpContext_t *zrtpContext, bzrtpChannelCon
 	/* flags */
 	zrtpChannelContext->isSecure = 0;
 	zrtpChannelContext->isMainChannel = isMain;
+#ifdef GOCLEAR_ENABLED
+	zrtpChannelContext->isClear = 0;
+	zrtpChannelContext->hasReceivedAGoClear = BZRTP_RECEPTION_UNKNOWN;
+#endif /* GOCLEAR_ENABLED */
 
 	/* initialise as initiator, switch to responder later if needed */
 	zrtpChannelContext->role = BZRTP_ROLE_INITIATOR;
@@ -1126,10 +1193,16 @@ static int bzrtp_initChannelContext(bzrtpContext_t *zrtpContext, bzrtpChannelCon
 	}
 	zrtpChannelContext->peerHelloHash = NULL;
 
+	/* initialisation of fragmented packet reception */
+	zrtpChannelContext->incomingFragmentedPacket.fragments = NULL;
+	zrtpChannelContext->incomingFragmentedPacket.messageId = 0;
+	zrtpChannelContext->incomingFragmentedPacket.packetString = NULL;
+
 	/* initialise the self Sequence number to a random and peer to 0 */
 	bctbx_rng_get(zrtpContext->RNGContext, (uint8_t *)&(zrtpChannelContext->selfSequenceNumber), 2);
 	zrtpChannelContext->selfSequenceNumber &= 0x0FFF; /* first 4 bits to zero in order to avoid reaching FFFF and turning back to 0 */
 	zrtpChannelContext->selfSequenceNumber++; /* be sure it is not initialised to 0 */
+	zrtpChannelContext->selfMessageSequenceNumber = zrtpChannelContext->selfSequenceNumber; /* message sequence number gets the same initial value */
 	zrtpChannelContext->peerSequenceNumber = 0;
 
 	/* reset choosen algo and their functions */
@@ -1176,7 +1249,7 @@ static int bzrtp_initChannelContext(bzrtpContext_t *zrtpContext, bzrtpChannelCon
 		return retval;
 	}
 	/* build the packet string and store the packet */
-	if (bzrtp_packetBuild(zrtpContext, zrtpChannelContext, helloPacket, zrtpChannelContext->selfSequenceNumber) ==0) {
+	if (bzrtp_packetBuild(zrtpContext, zrtpChannelContext, helloPacket) ==0) {
 		zrtpChannelContext->selfPackets[HELLO_MESSAGE_STORE_ID] = helloPacket;
 	} else {
 		bzrtp_freeZrtpPacket(helloPacket);
@@ -1207,27 +1280,7 @@ static void bzrtp_destroyChannelContext(bzrtpContext_t *zrtpContext, bzrtpChanne
 	/* set timer off */
 	zrtpChannelContext->timer.status = BZRTP_TIMER_OFF;
 
-	/* destroy and free the key buffers */
-	bzrtp_DestroyKey(zrtpChannelContext->s0, zrtpChannelContext->hashLength, zrtpContext->RNGContext);
-	bzrtp_DestroyKey(zrtpChannelContext->KDFContext, zrtpChannelContext->KDFContextLength, zrtpContext->RNGContext);
-	bzrtp_DestroyKey(zrtpChannelContext->mackeyi, zrtpChannelContext->hashLength, zrtpContext->RNGContext);
-	bzrtp_DestroyKey(zrtpChannelContext->mackeyr, zrtpChannelContext->hashLength, zrtpContext->RNGContext);
-	bzrtp_DestroyKey(zrtpChannelContext->zrtpkeyi, zrtpChannelContext->cipherKeyLength, zrtpContext->RNGContext);
-	bzrtp_DestroyKey(zrtpChannelContext->zrtpkeyr, zrtpChannelContext->cipherKeyLength, zrtpContext->RNGContext);
-
-	free(zrtpChannelContext->s0);
-	free(zrtpChannelContext->KDFContext);
-	free(zrtpChannelContext->mackeyi);
-	free(zrtpChannelContext->mackeyr);
-	free(zrtpChannelContext->zrtpkeyi);
-	free(zrtpChannelContext->zrtpkeyr);
-
-	zrtpChannelContext->s0=NULL;
-	zrtpChannelContext->KDFContext=NULL;
-	zrtpChannelContext->mackeyi=NULL;
-	zrtpChannelContext->mackeyr=NULL;
-	zrtpChannelContext->zrtpkeyi=NULL;
-	zrtpChannelContext->zrtpkeyr=NULL;
+	bzrtp_destroyKeyMaterial(zrtpContext, zrtpChannelContext);
 
 	/* free the allocated buffers */
 	for (i=0; i<PACKET_STORAGE_CAPACITY; i++) {
@@ -1239,18 +1292,11 @@ static void bzrtp_destroyChannelContext(bzrtpContext_t *zrtpContext, bzrtpChanne
 	free(zrtpChannelContext->peerHelloHash);
 	zrtpChannelContext->peerHelloHash = NULL;
 
-	/* destroy and free the srtp and sas struture */
-	bzrtp_DestroyKey(zrtpChannelContext->srtpSecrets.selfSrtpKey, zrtpChannelContext->srtpSecrets.selfSrtpKeyLength, zrtpContext->RNGContext);
-	bzrtp_DestroyKey(zrtpChannelContext->srtpSecrets.selfSrtpSalt, zrtpChannelContext->srtpSecrets.selfSrtpSaltLength, zrtpContext->RNGContext);
-	bzrtp_DestroyKey(zrtpChannelContext->srtpSecrets.peerSrtpKey, zrtpChannelContext->srtpSecrets.peerSrtpKeyLength, zrtpContext->RNGContext);
-	bzrtp_DestroyKey(zrtpChannelContext->srtpSecrets.peerSrtpSalt, zrtpChannelContext->srtpSecrets.peerSrtpSaltLength, zrtpContext->RNGContext);
-	bzrtp_DestroyKey((uint8_t *)zrtpChannelContext->srtpSecrets.sas, zrtpChannelContext->srtpSecrets.sasLength, zrtpContext->RNGContext);
-
-	free(zrtpChannelContext->srtpSecrets.selfSrtpKey);
-	free(zrtpChannelContext->srtpSecrets.selfSrtpSalt);
-	free(zrtpChannelContext->srtpSecrets.peerSrtpKey);
-	free(zrtpChannelContext->srtpSecrets.peerSrtpSalt);
-	free(zrtpChannelContext->srtpSecrets.sas);
+	/* free possible fragment management buffers */
+	bctbx_list_free_with_data(zrtpChannelContext->incomingFragmentedPacket.fragments, bctbx_free);
+	zrtpChannelContext->incomingFragmentedPacket.fragments = NULL;
+	bctbx_free(zrtpChannelContext->incomingFragmentedPacket.packetString);
+	zrtpChannelContext->incomingFragmentedPacket.packetString = NULL;
 
 	/* free the channel context */
 	free(zrtpChannelContext);
@@ -1265,3 +1311,168 @@ static uint8_t copyCryptoTypes(uint8_t destination[7], uint8_t source[7], uint8_
 	}
 	return size;
 }
+
+const char *bzrtp_algoToString(uint8_t algo){
+	switch(algo) {
+	case(ZRTP_UNSET_ALGO): return "unset";
+
+	case(ZRTP_HASH_S256): return "SHA-256";
+	case(ZRTP_HASH_S384): return "SHA-384";
+	case(ZRTP_HASH_S512): return "SHA-512";
+	case(ZRTP_HASH_N256): return "SHA3-256";
+	case(ZRTP_HASH_N384): return "SHA3-384";
+
+	case(ZRTP_CIPHER_AES1): return "AES-128";
+	case(ZRTP_CIPHER_AES2): return "AES-192";
+	case(ZRTP_CIPHER_AES3): return "AES-256";
+	case(ZRTP_CIPHER_2FS1): return "TwoFish-128";
+	case(ZRTP_CIPHER_2FS2): return "TwoFish-192";
+	case(ZRTP_CIPHER_2FS3): return "TwoFish-256";
+
+	case(ZRTP_AUTHTAG_HS32): return "HMAC-SHA1-32";
+	case(ZRTP_AUTHTAG_HS80): return "HMAC-SHA1-80";
+	case(ZRTP_AUTHTAG_SK32): return "Skein-32";
+	case(ZRTP_AUTHTAG_SK64): return "Skein-64";
+
+	case(ZRTP_KEYAGREEMENT_DH2k): return "DHM-2048";
+	case(ZRTP_KEYAGREEMENT_EC25): return "ECDH-256";
+	case(ZRTP_KEYAGREEMENT_DH3k): return "DHM-3072";
+	case(ZRTP_KEYAGREEMENT_EC38): return "ECDH-384";
+	case(ZRTP_KEYAGREEMENT_EC52): return "ECDH-521";
+	case(ZRTP_KEYAGREEMENT_X255): return "X25519";
+	case(ZRTP_KEYAGREEMENT_X448): return "X448";
+	case(ZRTP_KEYAGREEMENT_K255): return "K25519";
+	case(ZRTP_KEYAGREEMENT_K448): return "K448";
+	case(ZRTP_KEYAGREEMENT_Prsh): return "PreShared";
+	case(ZRTP_KEYAGREEMENT_Mult): return "MultiStream";
+	case(ZRTP_KEYAGREEMENT_KYB1): return "Kyber-512";
+	case(ZRTP_KEYAGREEMENT_KYB2): return "Kyber-768";
+	case(ZRTP_KEYAGREEMENT_KYB3): return "Kyber-1024";
+	case(ZRTP_KEYAGREEMENT_HQC1): return "HQC-128";
+	case(ZRTP_KEYAGREEMENT_HQC2): return "HQC-192";
+	case(ZRTP_KEYAGREEMENT_HQC3): return "HQC-256";
+	case(ZRTP_KEYAGREEMENT_K255_KYB512): return "K25519-Kyber512";
+	case(ZRTP_KEYAGREEMENT_K255_HQC128): return "K25519-HQC128";
+	case(ZRTP_KEYAGREEMENT_K448_KYB1024): return "K448-Kyber1024";
+	case(ZRTP_KEYAGREEMENT_K448_HQC256): return "K448-HQC256";
+	case(ZRTP_KEYAGREEMENT_K255_KYB512_HQC128): return "K25519-Kyber512-HQC128";
+	case(ZRTP_KEYAGREEMENT_K448_KYB1024_HQC256): return "K448-Kyber1024-HQC256";
+
+	case(ZRTP_SAS_B32): return "Base32";
+	case(ZRTP_SAS_B256): return "PGP-WordList";
+
+	default: return "Unknown Algo";
+	}
+}
+
+int bzrtp_set_MTU(bzrtpContext_t *zrtpContext, size_t mtu) {
+	if (zrtpContext == NULL) {
+		return BZRTP_ERROR_INVALIDCONTEXT;
+	}
+	if (mtu > BZRTP_MINIMUM_MTU) {
+		zrtpContext->mtu = mtu;
+	} else {
+		zrtpContext->mtu = BZRTP_MINIMUM_MTU;
+	}
+	return 0;
+}
+
+size_t bzrtp_get_MTU(bzrtpContext_t *zrtpContext) {
+	if (zrtpContext == NULL) {
+		return BZRTP_DEFAULT_MTU;
+	}
+	return zrtpContext->mtu;
+}
+
+int bzrtp_sendGoClear(bzrtpContext_t *zrtpContext, uint32_t selfSSRC){
+#ifdef GOCLEAR_ENABLED
+	/* get channel context */
+	bzrtpChannelContext_t *zrtpChannelContext = getChannelContext(zrtpContext, selfSSRC);
+
+	if (zrtpChannelContext == NULL && zrtpChannelContext->isSecure) {
+		return BZRTP_ERROR_INVALIDCONTEXT;
+	}
+
+	bzrtpEvent_t goClearEvent;
+
+	/* create a GoClear event */
+	goClearEvent.eventType = BZRTP_EVENT_GOCLEAR;
+	goClearEvent.bzrtpPacketString = NULL;
+	goClearEvent.bzrtpPacketStringLength = 0;
+	goClearEvent.bzrtpPacket = NULL;
+	goClearEvent.zrtpContext = zrtpContext;
+	goClearEvent.zrtpChannelContext = zrtpChannelContext;
+
+	/* send it to the state machine*/
+	if (zrtpChannelContext->stateMachine != NULL) {
+		return zrtpChannelContext->stateMachine(goClearEvent);
+	}
+
+	return 0;
+#else
+	return BZRTP_ERROR_GOCLEARDISABLED;
+#endif /* GOCLEAR_ENABLED */
+}
+
+int bzrtp_confirmGoClear(bzrtpContext_t *zrtpContext, uint32_t selfSSRC){
+#ifdef GOCLEAR_ENABLED
+	/* get channel context */
+	bzrtpChannelContext_t *zrtpChannelContext = getChannelContext(zrtpContext, selfSSRC);
+
+	if (zrtpChannelContext == NULL) {
+		return BZRTP_ERROR_INVALIDCONTEXT;
+	}
+
+	bzrtpEvent_t acceptGoClearEvent;
+
+	zrtpChannelContext->timer.firingCount++;
+
+	/* create a accept GoClear event */
+	acceptGoClearEvent.eventType = BZRTP_EVENT_ACCEPT_GOCLEAR;
+	acceptGoClearEvent.bzrtpPacketString = NULL;
+	acceptGoClearEvent.bzrtpPacketStringLength = 0;
+	acceptGoClearEvent.bzrtpPacket = NULL;
+	acceptGoClearEvent.zrtpContext = zrtpContext;
+	acceptGoClearEvent.zrtpChannelContext = zrtpChannelContext;
+
+	/* send it to the state machine */
+	if (zrtpChannelContext->stateMachine != NULL) {
+		return zrtpChannelContext->stateMachine(acceptGoClearEvent);
+	}
+
+	return 0;
+#else
+	return BZRTP_ERROR_GOCLEARDISABLED;
+#endif /* GOCLEAR_ENABLED */
+}
+
+int bzrtp_backToSecureMode(bzrtpContext_t *zrtpContext, uint32_t selfSSRC){
+#ifdef GOCLEAR_ENABLED
+	/* get channel context */
+	bzrtpChannelContext_t *zrtpChannelContext = getChannelContext(zrtpContext, selfSSRC);
+	bzrtpEvent_t backToSecureEvent;
+
+	if (zrtpChannelContext == NULL) {
+		return BZRTP_ERROR_INVALIDCONTEXT;
+	}
+
+	zrtpChannelContext->timer.firingCount++;
+
+	/* create a GoClear event */
+	backToSecureEvent.eventType = BZRTP_EVENT_BACKTOSECURE;
+	backToSecureEvent.bzrtpPacketString = NULL;
+	backToSecureEvent.bzrtpPacketStringLength = 0;
+	backToSecureEvent.bzrtpPacket = NULL;
+	backToSecureEvent.zrtpContext = zrtpContext;
+	backToSecureEvent.zrtpChannelContext = zrtpChannelContext;
+
+	/* send it to the state machine */
+	if (zrtpChannelContext->stateMachine != NULL) {
+		return zrtpChannelContext->stateMachine(backToSecureEvent);
+	}
+
+	return 0;
+#else
+	return BZRTP_ERROR_GOCLEARDISABLED;
+#endif /* GOCLEAR_ENABLED */
+}
